#include <qvbox.h>
#include <qlabel.h>
#include <qvbuttongroup.h>
#include <qhbuttongroup.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qsettings.h>
#include <qlayout.h>
#include <qdialog.h>
#include <qspinbox.h>
#include <qfile.h>
#include <qapplication.h>

#include "callaudio.h"
#include "dspoutoss.h"
#include "audiocontrol.h"

/*
CHANGELOG:

// CHANGED 2004-07-28 Lutz Rogowski
CHANGED 2004-07-19 Lutz Rogowski
*/

AudioControl::AudioControl( CallAudio *callaudio, QWidget *parent, const char *name )
	: QTabDialog( parent, QString( name ) + "Audio Properties", true )
{
    setCaption( "KP/Pi Audio Properties");
	a = callaudio;
	QVBox *vbox = new QVBox( );	
	vbox->setMargin( 3 );
	vbox->setSpacing( 3 );	
	addTab( vbox, "Device" );
	//QVBoxLayout *vboxl = new QVBoxLayout( this, 0 );
	//vboxl->addWidget( vbox );
	
	bg2 = new QHButtonGroup( "OSS Device Mode", vbox );
	rwButton = new QRadioButton( "ReadWrite", bg2 );
	bg2->insert( rwButton, rw );
	rowoButton = new QRadioButton( "ROnly/WOnly", bg2 );
	bg2->insert( rowoButton, rowo );
	connect( rwButton, SIGNAL( clicked() ), this, SLOT( rwButtonClicked() ) );
	connect( rowoButton, SIGNAL( clicked() ), this, SLOT( rowoButtonClicked() ) );

	(void) new QLabel( "Device for WriteOnly or ReadWrite:", vbox );
    QHBox *hbW = new QHBox(vbox );	
	dspdev = new QLineEdit( a->getOSSFilename(), hbW );
    QLabel* lab = new QLabel ( " FS: ", hbW );
    fsW = new QSpinBox( 6,10,1,hbW );
	//dspdev->setMinimumWidth( fontMetrics().maxWidth() * 20 );
	dspdev->setText( a->getOSSFilename() );

	(void) new QLabel( "Device for ReadOnly:", vbox );
    QHBox *hbR = new QHBox(vbox );	
	dspdev2 = new QLineEdit( a->getOSSFilename2(), hbR );
    lab = new QLabel ( " FS: ", hbR );
    fsR = new QSpinBox( 6,10,1,hbR );
	//dspdev2->setMinimumWidth( fontMetrics().maxWidth() * 20 );
	dspdev2->setText( a->getOSSFilename2() );
	vbox = new QVBox( );
	vbox->setMargin( 3 );
	vbox->setSpacing( 3 );		
	addTab( vbox, "Codec" );

	bg1 = new QHButtonGroup( "Preferred Codec", vbox );
	QRadioButton *ulawButton = new QRadioButton( "G711u", bg1 );
	bg1->insert( ulawButton, ulaw );
    ulawButton->setMaximumSize( ulawButton->sizeHint() );
	QRadioButton *gsmButton = new QRadioButton( "GSM", bg1 );
	bg1->insert( gsmButton, gsm );
    gsmButton->setMaximumSize(gsmButton->sizeHint() );
	QRadioButton *ilbcButton = new QRadioButton( "iLBC", bg1 );
	bg1->insert( ilbcButton, ilbc );
    //ilbcButton->setMaximumSize(ilbcButton->sizeHint() );
	QSettings settings;
	QString mode = settings.readEntry( "/kphone/audio/mode", "readwrite" );
	if( mode == "readwrite" ) {
		rwButton->setChecked(TRUE);
		dspdev2->setEnabled(FALSE);
        fsR->setEnabled(FALSE);
	} else {
		rowoButton->setChecked(TRUE);
		dspdev2->setEnabled(TRUE);
        fsR->setEnabled(TRUE);
	}
	
	QString codec = settings.readEntry( "/kphone/audio/Codec", "ULAW" );
	if( codec == "ULAW" ) {
		ulawButton->setChecked(TRUE);
	} else if( codec == "GSM" ) {
		gsmButton->setChecked(TRUE);
	} else if( codec == "ILBC" ) {
		ilbcButton->setChecked(TRUE);
	}

	// Size of Payload
	bg3 = new QVButtonGroup( "Size of Payload (G711u)", vbox );
	QRadioButton *s80 = new QRadioButton( " 80 ( 10 ms )", bg3 );
	bg3->insert( s80, size80 );
	QRadioButton *s160 = new QRadioButton( "160 ( 20 ms )", bg3 );
	bg3->insert( s160, size160 );
	QRadioButton *s240 = new QRadioButton( "240 ( 30 ms )", bg3 );
	bg3->insert( s240, size240 );
	switch( settings.readNumEntry( "/kphone/dsp/SizeOfPayload", 80 ) ) {
		case 80: s80->setChecked(TRUE); break;
		case 160: s160->setChecked(TRUE); break;
		case 240: s240->setChecked(TRUE); break;
	}
    int fs;
    fs = settings.readNumEntry( "/kphone/dsp/FragSizeWrite", 7 );
    fsW->setValue( fs );
    fs = settings.readNumEntry( "/kphone/dsp/FragSizeRead", 7 );
    fsR->setValue( fs );

	vbox = new QVBox( );
	vbox->setMargin( 3 );
	vbox->setSpacing( 3 );		
	addTab( vbox, "Ring tone" );
	// Ringing tone
	ringing = new QVButtonGroup( "Ringing tone ", vbox );
	QRadioButton *noRingingtone = new QRadioButton( "No ring tone", ringing );
	ringing->insert( noRingingtone, ringingtoneNo );
	QRadioButton *yesRingingtone = new QRadioButton( "Use system alarm", ringing );
	ringing->insert( yesRingingtone, ringingtoneYes );
	QRadioButton *wavRingingtone = new QRadioButton( "Use wav file", ringing );
	ringing->insert( wavRingingtone, ringingtoneWav );
    QString ringset = settings.readEntry( "/kphone/General/ringingtone", "No" ) ;
	if( ringset == "Yes" ) {
		ringing->setButton( ringingtoneYes );
	} else {
        if( ringset == "Wav" ) {
            ringing->setButton( ringingtoneWav );
        } else
            ringing->setButton( ringingtoneNo );
	}
    if ( QApplication::desktop()->width() <= 640 ) { 
        if ( QFile::exists ( "/dev/sharp_led" ) ) 
            new QLabel( "Alarm LED will always\nflash on Zaurus models.", vbox );

    }
	setCancelButton();
	connect( this, SIGNAL( cancelButtonPressed() ), this, SLOT( slotCancel() ) );
	setOKButton();
	connect( this, SIGNAL( applyButtonPressed() ), this, SLOT( slotOk() ) ); 
    //setHelpButton () ;
	//connect( this, SIGNAL( helpButtonPressed() ), parent, SLOT( showHelp() )); 
#if 0 
	QHBoxLayout *buttonBox;
	buttonBox = new QHBoxLayout( vboxl, 6 );
	okPushButton = new QPushButton( this, "ok button" );
	okPushButton->setText( "OK" );
	okPushButton->setDefault( TRUE );
	buttonBox->addWidget( okPushButton );
	cancelPushButton = new QPushButton( this, "cancel button" );
	cancelPushButton->setText( "Cancel" );
	cancelPushButton->setAccel( Key_Escape );
	buttonBox->addWidget( cancelPushButton );
	connect( okPushButton, SIGNAL( clicked() ), this, SLOT( slotOk() ) );
	connect( cancelPushButton, SIGNAL( clicked() ), this, SLOT( slotCancel() ) );
#endif
   
}

AudioControl::~AudioControl( void )
{
}

void AudioControl::rwButtonClicked( void )
{
	dspdev2->setEnabled(FALSE);
    fsR->setEnabled(false);
}

void AudioControl::rowoButtonClicked( void )
{
	dspdev2->setEnabled(TRUE);
    fsR->setEnabled(TRUE);
}

void AudioControl::slotOk( void )
{
	slotApply();
	QDialog::accept();
}

void AudioControl::slotApply( void )
{
	a->setOSSFilename( dspdev->text() );
	a->setOSSFilename2( dspdev2->text() );
	a->saveAudioSettings();
	QSettings settings;
    qDebug("AudioControl::slotApply ");
	switch( bg1->id( bg1->selected() ) ) {
		case ulaw:
			settings.writeEntry( "/kphone/audio/Codec", "ULAW");
			a->setCodec( codecULAW );
			break;
		case gsm:
			settings.writeEntry( "/kphone/audio/Codec", "GSM");
			a->setCodec( codecGSM );
			break;
		case ilbc:
			settings.writeEntry( "/kphone/audio/Codec", "ILBC");
			a->setCodec( codecILBC );
			break;
	}

	switch( bg2->id( bg2->selected() ) ) {
		case rw:
			settings.writeEntry( "/kphone/audio/mode", "readwrite");
			break;
		case rowo:
			settings.writeEntry( "/kphone/audio/mode", "readonly_writeonly");
			break;
	}

	int payload;
	switch( bg3->id( bg3->selected() ) ) {
		case size80:
			payload = 80;
			break;
		case size160:
			payload = 160;
			break;
		case size240:
			payload = 240;
			break;
		default:
			payload = 160;
			break;
	}
	a->setPayload( payload );
	settings.writeEntry( "/kphone/dsp/SizeOfPayload", payload );
    settings.writeEntry( "/kphone/dsp/FragSizeWrite", fsW->value() );
    settings.writeEntry( "/kphone/dsp/FragSizeRead", fsR->value() );
  
	switch( ringing->id( ringing->selected() ) ) {
		case ringingtoneYes:
			settings.writeEntry( "/kphone/General/ringingtone", "Yes" );
			break;
		case ringingtoneWav:
			settings.writeEntry( "/kphone/General/ringingtone", "Wav" );
			break;
		default:
			settings.writeEntry( "/kphone/General/ringingtone", "No" );
			break;
	}
}

void AudioControl::slotCancel( void )
{
	dspdev->setText( a->getOSSFilename() );
	dspdev2->setText( a->getOSSFilename2() );
	QDialog::reject();
}

VideoControl::VideoControl( CallAudio *callaudio,
	QWidget *parent, const char *name )
	: QDialog( parent, QString( name ) + "Video Properties", true )
{
    setCaption( "KP/Pi Video Properties");
	a = callaudio;
	QVBox *vbox = new QVBox( this );
	vbox->setMargin( 3 );
	vbox->setSpacing( 3 );
	QVBoxLayout *vboxl = new QVBoxLayout( this, 5 );
	vboxl->addWidget( vbox );
	QSettings settings;
	(void) new QLabel( "/kphone/video/Video SW:", vbox );
	QString videosw = settings.readEntry(
		"/kphone/video/videoSW", "/usr/local/bin/vic" );
	QString codec = settings.readEntry( "/kphone/video/codec", "h261" );
	videoSW = new QLineEdit( videosw, vbox );
	//videoSW->setMinimumWidth( fontMetrics().maxWidth() * 20 );
	bg1 = new QHButtonGroup( "Preferred Video Codec", vbox );
	QRadioButton *h261Button = new QRadioButton( "H261", bg1 );
	bg1->insert( h261Button, h261 );
	QRadioButton *h263Button = new QRadioButton( "H263", bg1 );
	bg1->insert( h263Button, h263 );
	if( codec == "h261" ) {
		h261Button->setChecked(TRUE);
	} else if( codec == "h263" ) {
		h263Button->setChecked(TRUE);
	}

	QHBoxLayout *buttonBox;
	buttonBox = new QHBoxLayout( vboxl, 6 );
	okPushButton = new QPushButton( this, "ok button" );
	okPushButton->setText( "OK" );
	okPushButton->setDefault( TRUE );
	buttonBox->addWidget( okPushButton );
	cancelPushButton = new QPushButton( this, "cancel button" );
	cancelPushButton->setText( "Cancel" );
	cancelPushButton->setAccel( Key_Escape );
	buttonBox->addWidget( cancelPushButton );
	connect( okPushButton, SIGNAL( clicked() ),
		this, SLOT( slotOk() ) );
	connect( cancelPushButton, SIGNAL( clicked() ),
		this, SLOT( slotCancel() ) );

}

VideoControl::~VideoControl( void )
{
}

void VideoControl::slotOk( void )
{
	slotApply();
	QDialog::accept();
}

void VideoControl::slotApply( void )
{
	a->setVideoSW( videoSW->text() );
	QSettings settings;
	settings.writeEntry( "/kphone/video/videoSW", videoSW->text() );
	switch( bg1->id( bg1->selected() ) ) {
		case h261:
			settings.writeEntry( "/kphone/video/codec", "h261");
			a->setVideoCodec( codecH261 );
			break;
		case h263:
			settings.writeEntry( "/kphone/video/codec", "h263");
			a->setVideoCodec( codecH263 );
			break;
	}
}

void VideoControl::slotCancel( void )
{
	QDialog::reject();
}
