
/*
CHANGELOG:

// CHANGED 2004-07-28 Lutz Rogowski
// CHANGED 2004-07-21 Lutz Rogowski
CHANGED 2004-07-19 Lutz Rogowski
*/

#include <stdio.h>
#include <stdlib.h>
#include <qtimer.h>
#include <qsettings.h>
#include <qtextbrowser.h>
#include <qmenubar.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qapplication.h>

#include "../dissipate2/sipuser.h"
#include "../dissipate2/sipclient.h"
#include "../dissipate2/sipcall.h"
#include "../dissipate2/sipregister.h"
#include "callaudio.h"
#include "audiocontrol.h"
#include "ksipregistrations.h"
#include "kphoneview.h"
#include "ksippreferences.h"
#include "ksipoptions.h"
#include "kphonebook.h"
#include "kphone.h"


KPhone::KPhone( unsigned int listenport, QString prefix, bool looseRoute, bool strictRoute ) : QMainWindow(0)
{
	QString uristr;
    
	if( Sip::getLocalAddress().isEmpty() ) {
        qDebug("Sip::getLocalAddress().isEmpty(). Exiting. ");
		exit(0);
	}
	userPrefix = "";
    setCaption( "KPhone/Pi" );
	QSettings settings;
	Sip::setLocalAddress( settings.readEntry(
		"/kphone/dissipate_addr", Sip::getLocalAddress() ) );
	QString socketStr = settings.readEntry( "/kphone/General/SocketMode", "UDP" );
	
	client = new SipClient( 0, 0, listenport, looseRoute, strictRoute, socketStr );
	QString p = "/kphone/" + getUserPrefix() + "Registration/";
	uristr = settings.readEntry( p + "SipUri" );

	if( settings.readEntry( p + "/UseProxyDial", "Yes" ) == "Yes" ) {
		client->setUseProxyDial( true );
	} else {
		client->setUseProxyDial( false );
	}
	client->setCallForward( settings.readBoolEntry( p + "/callforward", false ) );
	client->setCallForwardUri( SipUri( settings.readEntry( p + "/forwardaddr", QString::null ) ) );
	client->setCallForwardMessage( settings.readEntry( p + "/forwardmsg", QString::null ) );
	client->setMaxForwards( settings.readNumEntry( p + "/maxforwards", 0 ) );
	client->setBusyMessage( settings.readEntry( p + "/busymsg", QString::null ) );
	if( settings.readEntry( "/kphone/Symmetric/Signalling", "Yes" ) == "Yes" ) {
		client->setSymmetricMode( true );
	}
	QString hvstr = settings.readEntry( "/kphone/Symmetric/hideviamode", "NoHide" );
	if( hvstr == "NoHide" ) { client->setHideViaMode( SipClient::DontHideVia ); }
	else if( hvstr == "HideHop" ) { client->setHideViaMode( SipClient::HideHop ); }
	else if( hvstr == "HideRoute" ) { client->setHideViaMode( SipClient::HideRoute ); }
	user = new SipUser( client, SipUri( uristr ) );
	audio = new CallAudio;
	audio->readAudioSettings();
	audio->readVideoSettings();

	// Initially, none of the dialogs exist yet
	qdial = 0;
	userinfo = 0;
	audiocontrol = 0;
	videocontrol = 0;
	registrations = 0;
	sipprefs = 0;
	queryoptions = 0;
	testoptions = 0;
	view = new KPhoneView( audio, client, user, this, getUserPrefix() );
	if( uristr != QString::null ) {
		p = "/kphone/" + getUserPrefix() + "Registration/";
		QString userdefaultproxy = settings.readEntry( p + "SipServer", QString::null );
		if( userdefaultproxy.lower() == "sip:" ) {
			userdefaultproxy = "";
		}
		view->updateIdentity( user );
		client->updateIdentity( user, userdefaultproxy );
	}

	registrations = new KSipRegistrations( client, view, this, getUserPrefix() );
	view->identities( registrations );

	setCentralWidget( view );

	QPopupMenu *popup = new QPopupMenu( this );
	popup->insertItem( "New Call...", view, SLOT( makeNewCall() ) );
	popup->insertItem( "New Video Call...", view, SLOT( makeVideoCall() ) );
	popup->insertItem( "Identity...", this, SLOT( showRegistrations() ) );

//#test
//	popup->insertItem( "Options...", this, SLOT( showQueryOptions() ) );
//	popup->insertItem( "Test...", this, SLOT( showTestOptions() ) );

	popup->insertSeparator();
	popup->insertItem( "&Quit", this, SLOT( kphoneQuit() ) );
	menuBar()->insertItem( "&File", popup );
	popup = new QPopupMenu( this );
	popup->insertItem( "SIP Preferences...", this, SLOT( showGlobalPreferences() ) );
	popup->insertItem( "Audio Preferences...", this, SLOT( showAudioPreferences() ) );
	popup->insertItem( "Video Preferences...", this, SLOT( showVideoPreferences() ) );
	popup->insertItem( "Phone Book", view, SLOT( showPhoneBook() ) );
	menuBar()->insertItem( "&Preferences", popup );
	menuBar()->insertSeparator();
	popup = new QPopupMenu( this );
	popup->insertItem( "Audio Setup", this, SLOT( showAudioHowTo() ) );
	popup->insertItem( "HowTo use", this, SLOT( showHowtoUse() ) );
	popup->insertItem( "Start hints", this, SLOT( showHelp() ) );
	popup->insertItem( "About KPhone/Pi", this, SLOT( showAboutKPhone() ) );
	menuBar()->insertItem( "&Help", popup );

	if( registrations->getUseStun() ) {
		audio->setStunSrv( registrations->getStunSrv() );
		client->sendStunRequest( registrations->getStunSrv() );
		QTimer *stun_timer = new QTimer( this );
		connect( stun_timer, SIGNAL( timeout() ), this, SLOT( stun_timerTick() ) );
		p = "/kphone/" + getUserPrefix() + "Registration/";
		int timeout = settings.readNumEntry( p + "RequestPeriod", constStunRequestPeriod );
		if( timeout > 0 ) {
			stun_timer->start( timeout * 1000 );
		}
	}
	QTimer *timer = new QTimer( this );
	connect( timer, SIGNAL( timeout() ), this, SLOT( timerTick() ) );
	timer->start( 100 );
	resize( 230, 230 );
    closeMe = false;
}

KPhone::~KPhone( void )
{
    
    delete client;
    delete user;
    delete audio;

}
void KPhone::receive( const QCString& cmsg, const QByteArray& data )
{
    qDebug("KP: QCOP message received: %s ", cmsg.data()  ); 
    QString mess ( cmsg );
    if ( mess == "-newCall" ) {
       
        showMaximized();
        raise();
        return;
       
    }
    if ( mess.left(5) == "-ring" ) {
        QString dial = "sip:" +mess.mid( 6 ).stripWhiteSpace();
        qDebug("dial %s ",dial.latin1());
        view->makeNewCallNum( dial );
        showMaximized();
        raise();
        return;
       
    }

}
void KPhone::timerTick( void )
{
	client->doSelect( false );
}

void KPhone::stun_timerTick( void )
{
	client->sendStunRequest();
}

void KPhone::showUsers( void )
{
}

QString KPhone::getUserPrefix( void )
{
	return userPrefix;
}

void KPhone::showUserInfo( void )
{
}

void KPhone::showAudioPreferences( void )
{
	if( !audiocontrol ) {
		audiocontrol = new AudioControl( audio, this, getUserPrefix() );
	}
	audiocontrol->show();
}

void KPhone::showVideoPreferences( void )
{
	if( !videocontrol ) {
		videocontrol = new VideoControl( audio, this, getUserPrefix() );
	}
	videocontrol->show();
}

void KPhone::showRegistrations( void )
{
	registrations->showIdentity();
}

void KPhone::showGlobalPreferences( void )
{
	if( !sipprefs ) {
		sipprefs = new KSipPreferences( client, audio, this, getUserPrefix() );
	}
	sipprefs->show();
}

void KPhone::showQueryOptions( void )
{
	if( !queryoptions ) {
		queryoptions = new KSipOptions( user, this );
	}
	queryoptions->show();
}

void KPhone::showTestOptions( void )
{
	if( !testoptions ) {
		testoptions = new KTestOptions( client );
	}
	client->setTest( true );
	testoptions->show();
}
void KPhone::displayText( QString text ,QString cap  )
{
    QDialog dia( this, "name", true ); ;
    dia.setCaption( cap );
    QVBoxLayout* lay = new QVBoxLayout( &dia );
    lay->setSpacing( 3 );
    lay->setMargin( 3 );
    QTextBrowser tb ( &dia );
    lay->addWidget( &tb ); 
    tb.setText( text );
#ifdef DESKTOP_VERSION
    dia.resize( 640, 480);
#else
    dia.showMaximized();
#endif
    dia.exec();
}

void KPhone::showAudioHowTo( )
{
 QString cap, text;
    cap = "KP/Pi HowTo Configure Audio";
	text = "If you have problems with audio setup,\n"
        "please start KPhone/Pi from console\n"
        "( with command kppi )\n"
        "and you will get a detailed output when\n"
        "opening the audio device.\n"
        "\n"
        "KPhone will work on Zaurus SL-7x0 \n"
        "and on Zaurus 5500 with 2.4.18 Kernel.\n"
        "The Sharp Rom 3.1 or OZ 3.3.5 have\n"
        "such a Kernel.\n"
        "\n"
        "You have to configure your Audio settings in menu:\n"
        "Preferences - Audio Preferences...\n"
        "**************************************\n"
        "Device settings:\n"
        "For the Zaurus SL 760 you should choose\n"
        "   Mode :ReadWrite. Device /dev/dsp \n"
        "For the Zaurus 5500 you should choose\n"
        "   Mode :ROnlyWOnly.\n"
        "   Device for WriteOnly: /dev/dsp.  \n"
        "   Device for ReadOnly: /dev/dsp1.  \n"
        "For any other device, try \n"
        "   Mode :ReadWrite.\n"
        "   Device: /dev/dsp or /dev/audio\n"
        "The default value for FS (Frag Size)\n"
        "is 7. It is highly recommended, \n"
        "that you use 7 or 8 as values here.\n"
        "Maybe choosing 8 will give you better\n"
        "sound quality on slow devices.\n"
        "(But a higher delay)\n"
        "You have to try it out.\n"
        "Note: Increasing the FS setting by one,\n"
        "doubles the FragSize!\n"
        "**************************************\n"
        "Codec settings:\n"
        "G711u: \n"
        "  High Bandwidth, good sound quality,\n"
        "  may have minor realtime problems on slow devices\n"
        "  (like the Zaurus 5500).\n"
        "GSM:\n"
        "  Low Bandwidth, low  sound quality,\n"
        "  almost no realtime problems on slow devices.\n"
        "iLBC:\n"
        "  Low Bandwidth, better  sound quality.\n"
        "  Needs fast machines. \n"
        "  Zaurus is too slow for that codec.\n"
        "\n"
        "Size of Payload settings (for G711u codec):\n"
        "  Recommended settings:\n"
        "    80 (on fast machines) or 160.\n"
        "    On slow devices, you may try 240\n"
        "  If you get drop outs in the sound stream\n"
        "  you should increase the size.\n"
        "**************************************\n"
        "Ring tone settings:\n"
        "No ring tone:\n"
        "  Only a call window pops up, when called.\n"
        "Use system alarm\n"
        "  Uses the buzzer on the Zaurus models.\n"
        "Use wav file:\n"
        "  Plays back the file \n"
        "  (QTOPIADIR)/pics/kphone/koalarm.wav\n"
        "  [ Usually, QTOPIADIR= /opt/QtPalmtop ]\n"
        "Hint 1.)\n"
        "  If you want to change the ring tone, \n"
        "  exchange this file.\n"
        "Hint 2.)\n"
        "  If you choose \"Use wav file\"\n"
        "  on a Z 5500, you will hear the \n"
        "  ring tone only on \n"
        "  the headset connected to your Z.\n"
        "\n";

    displayText( text,cap );
}

void KPhone::showHowtoUse()
{
    //receive( QCString( "-ring:+4943136889"), QByteArray() );

    //return;
  QString cap, text;
    cap = "KPhone/Pi HowTo use";
	text = "KPhone/Pi is a 'Voice Over Internet' phone\n"
        "which uses the Session Initiation Protocol (SIP).\n"
        "SIP services are something like index or catalogue services\n"
        "which are mapping a UserID to a real telephone number \n"
        "or to the IP address of the client.\n"
        "Such services are available as Open Source.\n"
        "\n"
        "There are provider, which offer such SIP services\n"
        "and offer telephone calls to the phone net\n"
        "via an \"IP to telephone gateway\".\n"
        "To use these services, you need a hardware IP telephone\n"
        "or a softphone like KPhone/Pi.\n"
        "\n"
        "We will now describe how to configure KPhone/Pi\n"
        "to use with the german SIP servive \"sipgate.de\" as an example:\n"
        "\n"
        "1) Register at sipgate.de and get an account. On 2004-07-18 this is for free.\n"
        "   You have to pay only, if you want to make calls to the real telephone net.\n"
        "   You will get a telephone number of the real telephone net.\n"
        "   Note, that it may take some time after registration, until this number is available for calling.\n"
        "2) Login in your accout and choose \"meine Daten\" and \"Konfigurationsdaten anzeigen\" \n"
        "   You will see: (We use example values here for ID and Password)\n"
        "      SIP-ID:         1234321	\n"
        "      SIP-Passwort:   SiPpAsSwOrD\n"
        "      Registry:  	   sipgate.de\n"
        "      Proxy:          sipgate.de\n"
        "      STUN:           stun.sipgate.net:10000\n"
        "      NTP:            ntp.sipgate.net\n"
        "3) Start KPhone/Pi - the identity dialog pops up.\n"
        "   Fill in: (something in <> indicates, that you have to fill in fitting values)\n"
        "     Full Name: <your name>\n"
        "     User Part of SIP URL: 1234321\n"
        "     Host Part of SIP URL: sipgate.de\n"
        "     Outbound Proxy: sipgate.de\n"
        "     Authentication Username: 1234321\n"
        "     Choose Auto Register\n"
        "   Click OK\n"
        "   The password dialog pops up. Fill in SiPpAsSwOrD\n"
        "   If you are behind a Router\n"
        "       [ to be more precise: behind a NAT (Network Address Translation) ]\n"
        "       choose:\n"
        "       Menu: Preferences-SIP Preferences-TAB Socket\n"
        "         Set there Use STUN Server to YES\n"
        "         Click OK\n"
        "       Reopen Menu: Preferences-SIP Preferences-TAB STUN Server\n"
        "         Set there STUN Server to stun.sipgate.net:10000 \n"
        "         Click OK\n"
        "4) Close KPhone/Pi and wait some seconds\n"
        "5) Restart KPhone/Pi\n"
        "   You are asked for your password\n"
        "   Type in SiPpAsSwOrD\n"
        "6) Now you should be connected!\n"
        "   The icon in KPhone in the lower left corner should show the \"connected\" icon.\n"
        "   After logging off and in again at your sipgate.de account web page,\n"
        "   the state of your phone should show: Telefon Online: online\n"
        "7) Go offline by choosing menu: File-Identity: Button Unregister\n"
	    "8) Your settings are stored in the file:\n"
        "   (HOMEDIR)/kdepim/config/kphone/<yourprofilename>/kphonerc!\n"
        "9) HINT: If you do not want to be asked for the password at every startup,\n"  
        "   change in the file (HOME)/kdepim/config/<yourprofilename>/kphonerc the line\n"
        "   SipUri=\"Your Name\" <sip:1234321@sipgate.de>\n"
        "   to\n"
        "   SipUri=\"Your Name\" <sip:1234321:SiPpAsSwOrD@sipgate.de>\n"
;
    displayText( text,cap );
}
void KPhone::showHelp( void )
{
    QString cap, text;
    cap = "KPhone/Pi quick start hints";
	text = 
	"Please read >>HowTo use<< first, if KPhone/Pi is new to you!\n"
	"Your settings are stored in the file: (HOMEDIR)/kdepim/config/<yourprofilename>/kphonerc!\n"
		"When starting KPhone for the first time, you should enter your identity information in the window appearing.\n"
		"The information required for successful sipping are 'Full Name', 'User part of SIP URL' and 'Host part of SIP URL'.\n"
		"Note for KPhone 3.xx users: copy '~/.kde/share/config/kphonerc' file to (HOMEDIR)/kdepim/config/<yourprofilename>/ directory.\n"
		"\n"
		"If you have a NAPTR/SRV enabled DNS-server you can leave the 'Outbound Proxy' and 'Authentication Username' fields empty,\n"
		"as KPhone will automatically search for the server information by the 'Host part of SIP URL' and prompt the username/password.\n"
		"\n"
		"Mark the auto register selection if you wish KPhone to automatically register your identity on application start.\n"
		"If you want to use multiple identities, start KPhone with the '-u' option and the username of your selection.\n"
		"For example:  'kphone -u jsmith'.\n"
		"\n"
		"If you are behind a NAT (Network Address Translation) device, you should go to Preferences -> SIP Preferences -> Socket and\n"
		"check the 'Use STUN Server'\n"
		"\n"
		"The right button of your mouse is your best friend in KPhone's main UI. By clicking the right mouse button (RMB) in the main\n"
		"window you get a menu including all important functions of KPhone. You should edit the Phone Book and enter all your favourite\n"
		"contacts there. Of course this is not mandatory but it increases your efficiency with KPhone.\n"
		"\n"
		"When adding contacts to your Phone Book, simply click Add and enter the person's real name, SIP identity and the optional\n"
		"description. If you want your contacts to be displayed in the main UI, just check the 'Add Identity ...' box. After editing your\n"
		"contacts you can return to the main UI by clicking Hide. All your selected contacts should now appear there. Making calls and \n"
		"sending instant messages to your contacts is easy with the RMB. Just click and select the function which you'd like to initiate.\n";
    displayText( text,cap );


}

void KPhone::showAboutKPhone( void )
{
    QString cap, text;
    cap = "About KPhone/Pi";
	text =		"KPhone/Pi 1.0.0 is a 'Voice Over Internet' phone\n\n"
		"Copyright (c) 2004 Lutz Rogowski <lutz@pi-sync.net>\n"
		"KPhone/Pi is based on KPhone " + KPhoneVersion +"\n"
		"Copyright (c) 2000 Billy Biggs <bbiggs@div8.net>\n"
		"Copyright (c) 2004 Wirlab <kphone@wirlab.net>\n\n"
		"KPhone/Pi is released under the GPL\n\n"
		"For more information, see\n"
		"www.wirlab.net/kphone\n"
		"www.pi-sync.net\n"
		"\n"
		"Part 10 and 11 of the TERMS AND CONDITIONS of the GPL:\n"
		"\n"
        " NO WARRANTY\n"
        "  11. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY\n"
        "FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN\n"
        "OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES\n"
        "PROVIDE THE PROGRAM \"AS IS\" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED\n"
        "OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF\n"
        "MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS\n"
        "TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE\n"
        "PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,\n"
        "REPAIR OR CORRECTION.\n"
        
        "  12. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING\n"
        "WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR\n"
        "REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,\n"
        "INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING\n"
        "OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED\n"
        "TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY\n"
        "YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER\n"
        "PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE\n"
        "POSSIBILITY OF SUCH DAMAGES.\n";

    displayText( text,cap );
}

void KPhone::kphoneQuit( void )
{
	view->kphoneQuit();
	hide(); 
    closeMe = true;
    QTimer::singleShot ( quitTime, this, SLOT( close()) ); 
}

void KPhone::closeEvent( QCloseEvent* ce )
{
    //qDebug("KPhone::closeEvent ");
    if ( closeMe ) {
        // we have to set closeMe to false, maybe fastload is enabled; 
        closeMe = false; 
        ce->accept(); 
        return;
    }
    if ( view->isCalling() ) {
        switch( QMessageBox::information( this, "KP/Pi",
                                      ("Phone call detected!\nDo you really want\nto close KPhone/Pi\nand hang up all Calls?"),
                                      ("Close"), ("No"), 
                                      0, 0 ) ) {
        case 0:
            break;
        case 1:
            ce->ignore(); 
            return;
            break;
        case 2:
        default:
            break;
        }



    }
    ce->ignore(); 
    kphoneQuit();
}
