
/*
CHANGELOG:

CHANGED 2004-07-19 Lutz Rogowski
*/

#ifndef KPHONE_H_INCLUDED
#define KPHONE_H_INCLUDED

#include <qmainwindow.h>

class SipClient;
class SipUser;
class PhoneList;
class CallAudio;
class AudioControl;
class VideoControl;
class KSipUserInfo;
class KPhoneView;
class KSipRegistrations;
class KSipAuthentication;
class KSipPreferences;
class KSipOptions;
class SipRegister;
class KTestOptions;

QString const KPhoneVersion = "4.0.3";
int const quitTime = 2000;
int const ringTime_1 = 500;
int const ringTime_2 = 5000;
int const acceptTime = 500;
int const acceptSubscribeSendToSubscribe_time = 5000;
int const constRegistrationExpiresTime = 900;
int const constSubscribeExpiresTime = 600;
int const constStunRequestPeriod = 60;
int const constMinPort = 0;
int const constMaxPort = 0;
int const kphoneMinimunWidht = 220;
QString const constStunServer = "stun.wirlab.net:3478";

extern QString globalIconPath;
#define SHARE_DIR globalIconPath +

extern QString globalProfilePath;
#include <qdir.h>
#include <qhbox.h>
#include <qlabel.h>
#include <qdialog.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qcombobox.h>
class KPProfilePrefs : public QDialog
{
    Q_OBJECT
        public:
    KPProfilePrefs( QWidget *par=0, const char *name=0 ) :
        QDialog( par, name, true ) 
        {
            setCaption( ("KP/Pi Profile Options") );
            QVBoxLayout* alarmLayout = new QVBoxLayout( this );
            alarmLayout->setSpacing( 3 );
            alarmLayout->setMargin( 3 );
            QWidget *parent = this;
            mSelectedDir = QDir::homeDirPath() + globalProfilePath+"kphone";
            QLabel* label = new QLabel( "<b>Choose profile or create new:</b>", parent);
            label->setAlignment( AlignCenter );
            alarmLayout->addWidget(label);
            mAlarmIncrCombo = new QComboBox(false, parent);

            QDir d ( mSelectedDir );
            QStringList sub = d.entryList(QDir::Dirs);
            QStringList subdirs;
            int i;
            for ( i = 0; i < sub.count(); ++i ) {
                QString mem = sub[i];
                if ( mem.left(1) != "." )
                    subdirs.append( mem );
            }
            mAlarmIncrCombo->insertStringList ( subdirs );

            //mAlarmIncrCombo->insertItem(i18n("minute(s)"));
            alarmLayout->addWidget(mAlarmIncrCombo);
            QPushButton * ok = new QPushButton( ("Continue"), this );
            if ( !mAlarmIncrCombo->count() )
                ok->setEnabled( false );
            alarmLayout->addWidget( ok );
            QHBox * hb = new QHBox ( parent );
            alarmLayout->addWidget(hb);
            mNewProfileName = new QLineEdit( hb );
            mAlarmSoundButton = new QPushButton("New", hb);
            connect(mAlarmSoundButton, SIGNAL(clicked()), SLOT(pickAlarmSound()));
            mAlarmSoundButton->setMaximumWidth( mAlarmSoundButton->sizeHint().width() );
            label = new QLabel( "Delete or rename profile by\nediting the directory name in\n"+mSelectedDir, parent);
            label->setAlignment( AlignCenter );
            alarmLayout->addWidget(label);
            QPushButton * cancel = new QPushButton( ("Cancel KP/Pi"), this );
            alarmLayout->addWidget( cancel );
            connect ( ok,SIGNAL(clicked() ),this , SLOT ( ok() )  );
            connect (cancel, SIGNAL(clicked() ), this, SLOT ( reject())  );
            //resize( 230, 230 );
     
        }
        QString mSelectedDir;
        QLineEdit * mNewProfileName;
        QComboBox* mAlarmIncrCombo ;
        QPushButton* mAlarmSoundButton ;

        private slots:
        void accept()
        {
            if ( !mNewProfileName->text().isEmpty() )
                pickAlarmSound();
            else if ( mAlarmIncrCombo->count() > 0 )
                ok();
        }
        void pickAlarmSound()
        {
            if ( mNewProfileName->text().isEmpty() )
                return;

            QString np = mSelectedDir + "/"+ mNewProfileName->text();
            QDir d ( np );
            if ( d.exists())
                return;
            d.mkdir( np );
            globalProfilePath += "kphone/"+mNewProfileName->text();
            mSelectedDir = mNewProfileName->text();
            QDialog::accept();
  
        }
        void ok()
        {
            globalProfilePath += "kphone/"+mAlarmIncrCombo->currentText();
            mSelectedDir = mAlarmIncrCombo->currentText();
            QDialog::accept();
        }
};


class KPhone : public QMainWindow
{
	Q_OBJECT
public:
	KPhone( unsigned int listenport, QString prefix, bool looseRoute, bool strictRoute );
	~KPhone( void );
	void showUsers( void );
	QString getUserPrefix( void );

public slots:
  void showHelp( void );
    void receive( const QCString& msg, const QByteArray& data );

private slots:
	void timerTick( void );
	void stun_timerTick( void );
	void showUserInfo( void );
	void showAudioPreferences( void );
	void showVideoPreferences( void );
	void showRegistrations( void );
	void showGlobalPreferences( void );
	void showQueryOptions( void );
	void showTestOptions( void );
    void showAboutKPhone( void );
    void showHowtoUse( void );
    void showAudioHowTo( void );
	void kphoneQuit( void );

protected:
    void displayText( QString text ,QString cap  );
	void closeEvent( QCloseEvent* );
	KPhoneView *view;

	// Global state
	SipClient *client;
	SipUser *user;
	CallAudio *audio;

	// Dialogs
	PhoneList *qdial;
	KSipUserInfo *userinfo;
	AudioControl *audiocontrol;
	VideoControl *videocontrol;
	KSipRegistrations *registrations;
	KSipAuthentication *authentication;
	KSipPreferences *sipprefs;
	KSipOptions *queryoptions;
	KTestOptions *testoptions;

private:
	QTimer *quitTimer;
	QString userPrefix;
    bool closeMe;

};

#endif // KPHONE_H_INCLUDED
