
/*
CHANGELOG:

// CHANGED 2004-07-21 Lutz Rogowski
CHANGED 2004-07-19 Lutz Rogowski
*/


#ifndef DESKTOP_VERSION
#include <qpe/qpeapplication.h>
#else
#include <qapplication.h>
#include <qstring.h>
#include <qwindowsstyle.h> 
#include <qplatinumstyle.h> 
#include <qsgistyle.h> 
#endif
#include "kphone.h"
#include "dtmfgenerator.h"
#include <stdlib.h>
#include <fcntl.h>



QString globalIconPath;
int global_fd_led = -1;

QString globalProfilePath = "/kdepim/config/";



int main( int argc, char **argv )
{
#ifdef DESKTOP_VERSION
    QApplication a( argc, argv ); 
    QApplication::setStyle( new QPlatinumStyle ());
    globalIconPath = qApp->applicationDirPath () + "/kdepim"; 
#else
	QPEApplication a( argc, argv );
    a.setKeepRunning ();
    globalIconPath = QString (getenv( "QTDIR" ) )+ "/pics"; 
#endif
    if ( QFile::exists ( "/dev/sharp_led" ) ) 
        global_fd_led = open ( "/dev/sharp_led", O_RDWR|O_NONBLOCK );
    // our hacked QSettings stores at homeDirPath() + /kdepim/config
    // such that we have to create the dirs
    QString setPath = QDir::homeDirPath() + "/kdepim";
    QDir setDir (setPath);
    if  ( ! setDir.exists () )
        setDir.mkdir ( setPath );
    if  ( ! setDir.exists (setPath + "/apps") )
        setDir.mkdir ( setPath + "/apps");
    if  ( ! setDir.exists (setPath + "/apps/kphone") )
        setDir.mkdir ( setPath + "/apps/kphone");
    setPath += "/config";
    if  ( ! setDir.exists (setPath) )
        setDir.mkdir ( setPath );
    setPath += "/kphone";
    if  ( ! setDir.exists (setPath) )
        setDir.mkdir ( setPath );
    
	int listenport = 5060;
	QString port = "";
	QString user = "";
	bool looseRoute = false;
	bool strictRoute = false;
    QString profile;
    {
        KPProfilePrefs pref;
        QPixmap pixi ( globalIconPath+"/kphone/kphone_large.png" );
        pref.setIcon( pixi );
        int res;
        res = pref.exec();
        if ( res != QDialog::Accepted ) {
            qDebug("cancel ");
            return -1;
        }
        profile =  pref.mSelectedDir;
    }
	for ( int i = 0; i < a.argc(); i++ ) {
		if( QString( a.argv()[i] ) == "-u" ) {
			if( i+1 < a.argc() ) {
				user = QString( a.argv()[++i] );
			}
		} else if( QString( a.argv()[i] ) == "-p" ) {
			if( i+1 < a.argc() ) {
				port = QString( a.argv()[++i] );
			}
			listenport = port.toInt();
		} else if( QString( a.argv()[i] ) == "-l" ) {
			looseRoute = true;
		} else if( QString( a.argv()[i] ) == "-s" ) {
			strictRoute = true;
		}
	}
    DTMFGenerator* dummyForCreateSamples = new DTMFGenerator();
    if ( dummyForCreateSamples->generateSamples() ) {
        KPhone phone ( listenport, user, looseRoute, strictRoute );
#ifdef DESKTOP_VERSION
        a.setMainWidget( &phone );
        phone.show();
        QObject::connect( &a, SIGNAL(lastWindowClosed ()), &a, SLOT( quit() )); 
#else   
        QObject::connect( &a, SIGNAL (appMessage ( const QCString &, const QByteArray & )),&phone, SLOT(receive(  const QCString&, const QByteArray& )));
        a.showMainWidget ( &phone );
#endif
        delete dummyForCreateSamples;
        phone.setCaption( "KP/Pi - " + profile  );
#ifdef DESKTOP_VERSION
        QPixmap pixi ( globalIconPath+"/kphone/kphone_large.png" );
        phone.setIcon( pixi );
#endif
        a.exec();
        DTMFGenerator::deleteSamples();
    } else {
        delete dummyForCreateSamples;
    }
    qDebug("KP/Pi: Bye! ");
}
